/*
{*******************************************************************}
{                                                                   }
{          BDB 2007                                                 }
{          Bain DB Builder                                          }
{                                                                   }
{          author: rain                                             }
{          date created: 2007/09/14                                 }
{          last update: 2007/09/15                                  }
{          remark: N/A                                              }
{                                                                   }
{          Ȩ(c) 2006-2007                            }
{          HTTP://WWW.BAINSOFT.COM                                  }
{          Copyright (c) 2006-2007 BAIN SOFTWARE                    }
{          ALL RIGHTS RESERVED                                      }
{                                                                   }
{*******************************************************************}
*/

using System;
using System.Xml;
using System.IO;
using System.Collections.Generic;

namespace BDB
{

    public class ResItemInfo
    {
        public bool Valid;
        public string Name;
    }

    public class ResTableHeaderInfo : ResItemInfo
    {
        public int ResIndex;
        public string DisplayName;
        public string HeaderSign;
        public int ItemCount;
        public int IndexCount;
        public int ConstraintCount;
        public string PKName;
        public string PK;
        public string Version;
        public string CreateSign;
        public string CreateTime;
        public string LastUpdateSign;
        public string LastUpdateTime;

        public void Clear()
        {
            Name = "";
            DisplayName = "";
            Valid = true;
            HeaderSign = "";
            ItemCount = 0;
            IndexCount = 0;
            ResIndex = -1;
            PKName = "";
            PK = "";
            Version = "";
            CreateSign = "";
            CreateTime = "";
            LastUpdateSign = "";
            LastUpdateTime = "";
        }

    }

    public class ResRowInfo : ResItemInfo
    {
        public string DisplayName;
        public bool IsPK;
        public int Index;
        public string TypeName;
        public int Precision;
        public int Scale;
        public int Length;
        public bool AllowNull;
        public bool Identity;
        public bool Visible;
        public string Default;
        public string Remark;
        public string OldColumnName;
        public int DisplayWidth;
        public string PickList;
        public string Source;
        public string SourceItems;
        public string DestItems;
        public string Reserved1;
        public string Reserved2;
        public string Reserved3;
        public string Reserved4;
        public string Reserved5;
        public string Reserved6;
        public string Reserved7;
    }

    public class ResIndexInfo : ResItemInfo
    {
        public string TableName;
        public bool PrimaryKey;
        public bool Constraint;
        public bool Unique;
        public bool Clustered;
        public int IndexType;
        public bool AutoUpdate;
        public int NullCollation;
        public int OrdinalPosition;
        public string ColumnName;
        public int Collation;
        public int Cardinality;
        public bool Integrated;
        public string Remark;
    }

    public class ResRelationInfo : ResItemInfo
    {
        public const string RSCascade = "CASCADE";

        public string TableName;
        public string PKTableName;
        public string PKColumnName;
        public string FKTableName;
        public string FKColumnName;
        public int Ordinal;
        public string UpdateRule;
        public string DeleteRule;
        public string PKName;
        public int Deferrability;
        public string Remark;

        public bool RelationUpdate()
        {
            return UpdateRule.Equals(RSCascade);
        }

        public bool RelationDelete()
        {
            return DeleteRule.Equals(RSCascade);
        }
    }

    public class ResTableInfo
    {
        public ResTableHeaderInfo HeaderInfo = new ResTableHeaderInfo();
        public List<ResRowInfo> RowList = new List<ResRowInfo>();
        public List<ResIndexInfo> IndexList = new List<ResIndexInfo>();
        public List<ResRelationInfo> RelationList = new List<ResRelationInfo>();

        public string TableName { get { return getTableName(); } }
        public int RowCount { get { return getRowCount(); } }
        public int IndexCount { get { return getIndexCount(); } }
        public int RelationCount { get { return getRelationCount(); } }

        private int getRowCount()
        {
            return RowList.Count;
        }

        private int getIndexCount()
        {
            return IndexList.Count;
        }

        private int getRelationCount()
        {
            return RelationList.Count;
        }

        private string getTableName()
        {
            return HeaderInfo.Name;
        }

        public virtual void Clear()
        {
            HeaderInfo.Clear();
            RowList.Clear();
            IndexList.Clear();
            RelationList.Clear();
        }
    }

    public class ResController
    {
        public const string RSAX = "AX";
        public const string RSTable = "TABLE";
        public const string RSRows = "ROWS";
        public const string RSRow = "ROW";
        public const string RSIndexs = "TABLEINDEXS";
        public const string RSConstraints = "TABLECONSTRAINTS";
        public const string RSName = "NAME";
        public const string RSValid = "VALID";
        public const string RSVisible = "VISIBLE";
        public const string RSRemark = "REMARK";
        public const string RSTableName = "TABLENAME";

        //row
        public const string RSResIndex = "RESINDEX";
        public const string RSDisplayName = "DISPLAYNAME";
        public const string RSHeaderSign = "HEADERSIGN";
        public const string RSItemCount = "ITEMCOUNT";
        public const string RSIndexCount = "INDEXCOUNT";
        public const string RSConstraintCount = "CONSTRAINTCOUNT";
        public const string RSPKName = "PKNAME";
        public const string RSPK = "PK";
        public const string RSVersion = "VERSION";
        public const string RSCreateSign = "CREATESIGN";
        public const string RSCreateTime = "CREATETIME";
        public const string RSLastUpdateSign = "LASTUPDATESIGN";
        public const string RSLastUpdateTime = "LASTUPDATETIME";
        public const string RSIsPK = "ISPK";
        public const string RSIndex = "INDEX";
        public const string RSTypeName = "TYPENAME";
        public const string RSPrecision = "PRECISION";
        public const string RSLength = "LENGTH";
        public const string RSAllowNull = "ALLOWNULL";
        public const string RSIdentity = "IDENTITY";
        public const string RSDefault = "DEFAULT";
        public const string RSOldColumnName = "OLDCOLUMNNAME";
        public const string RSDisplayWidth = "DIPLAYWIDTH";
        public const string RSPickList = "PICKLIST";
        public const string RSSource = "SOURCE";
        public const string RSSourceItems = "SOURCEITEMS";
        public const string RSDestItems = "DESTITEMS";
        public const string RSReserved1 = "RESERVED1";
        public const string RSReserved2 = "RESERVED2";
        public const string RSReserved3 = "RESERVED3";
        public const string RSReserved4 = "RESERVED4";
        public const string RSReserved5 = "RESERVED5";
        public const string RSReserved6 = "RESERVED6";
        public const string RSReserved7 = "RESERVED7";

        //index
        public const string RSIndexName = "INDEXNAME";
        public const string RSPrimaryKey = "PRIMARYKEY";
        public const string RSConstraint = "CONSTRAINT";
        public const string RSUnique = "UNIQUE";
        public const string RSClustered = "CLUSTERED";
        public const string RSIndexType = "INDEXTYPE";
        public const string RSAutoUpdate = "AUTOUPDATE";
        public const string RSNullCollation = "NULLCOLLATION";
        public const string RSOrdinalPosition = "ORDINALPOSITION";
        public const string RSColumnName = "COLUMNNAME";
        public const string RSCollation = "COLLATION";
        public const string RSCardinality = "CARDINALITY";
        public const string RSIntegrated = "INTEGRATED";

        //relation
        public const string RSPKTableName = "PKTABLENAME";
        public const string RSPKColumnName = "PKCOLUMNNAME";
        public const string RSFKTableName = "FKTABLENAME";
        public const string RSFKColumnName = "FKCOLUMNNAME";
        public const string RSOrdinal = "ORDINAL";
        public const string RSUpdateRule = "UPDATERULE";
        public const string RSDeleteRule = "DELETERULE";
        public const string RSFKName = "FKNAME";
        public const string RSDeferrability = "DEFERRABILITY";

        private static bool ParseStrToBool(string s)
        {
            if (s.Equals("1"))
                return true;
            return false;
        }

        private static int ParseStrToInt(string s)
        {
            if ((s == null) || s.Equals(""))
                return 0;
            return int.Parse(s);
        }

        public static void ResRead(string resFile, ResTableInfo TableInfo)
        {
            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.Load(resFile);

            if (xmlDoc.ChildNodes.Count == 0)
                return;

            XmlNode xnRoot = xmlDoc.SelectSingleNode(RSAX);

            if (xnRoot == null)
                return;

            XmlElement xeTable = (XmlElement)xnRoot.SelectSingleNode(RSTable);

            if (xeTable == null)
                return;

            ResTableHeaderInfo HeaderInfo = TableInfo.HeaderInfo;

            //header info
            HeaderInfo.Name = xeTable.GetAttribute(RSName);
            HeaderInfo.Valid = ParseStrToBool(xeTable.GetAttribute(RSValid));
            HeaderInfo.ResIndex = ParseStrToInt(xeTable.GetAttribute(RSResIndex));
            HeaderInfo.DisplayName = xeTable.GetAttribute(RSDisplayName);
            HeaderInfo.HeaderSign = xeTable.GetAttribute(RSHeaderSign);
            HeaderInfo.ItemCount = ParseStrToInt(xeTable.GetAttribute(RSItemCount));
            HeaderInfo.IndexCount = ParseStrToInt(xeTable.GetAttribute(RSIndexCount));
            HeaderInfo.ConstraintCount = ParseStrToInt(xeTable.GetAttribute(RSConstraintCount));
            HeaderInfo.PKName = xeTable.GetAttribute(RSPKName);
            HeaderInfo.PK = xeTable.GetAttribute(RSPK);
            HeaderInfo.Version = xeTable.GetAttribute(RSVersion);
            HeaderInfo.CreateSign = xeTable.GetAttribute(RSCreateSign);
            HeaderInfo.CreateTime = xeTable.GetAttribute(RSCreateTime);
            HeaderInfo.LastUpdateSign = xeTable.GetAttribute(RSLastUpdateSign);
            HeaderInfo.LastUpdateTime = xeTable.GetAttribute(RSLastUpdateTime);

            //rows
            XmlElement xeRows = (XmlElement)xeTable.SelectSingleNode(RSRows);
            if (xeRows != null)
            {

                XmlNodeList xnlRows = xeRows.ChildNodes;

                foreach (XmlNode xn in xnlRows)
                {
                    XmlElement xe = (XmlElement)xn;

                    ResRowInfo rowInfo = new ResRowInfo();

                    rowInfo.Name = xe.GetAttribute(RSName);
                    rowInfo.Valid = ParseStrToBool(xe.GetAttribute(RSValid));
                    rowInfo.Visible = ParseStrToBool(xe.GetAttribute(RSVisible));
                    rowInfo.Remark = xe.GetAttribute(RSRemark);
                    rowInfo.IsPK = ParseStrToBool(xe.GetAttribute(RSIsPK));
                    rowInfo.Index = ParseStrToInt(xe.GetAttribute(RSIndex));
                    rowInfo.TypeName = xe.GetAttribute(RSTypeName);
                    rowInfo.Precision = ParseStrToInt(xe.GetAttribute(RSPrecision));
                    rowInfo.Length = ParseStrToInt(xe.GetAttribute(RSLength));
                    rowInfo.AllowNull = ParseStrToBool(xe.GetAttribute(RSAllowNull));
                    rowInfo.Identity = ParseStrToBool(xe.GetAttribute(RSIdentity));
                    rowInfo.Default = xe.GetAttribute(RSDefault);
                    rowInfo.OldColumnName = xe.GetAttribute(RSOldColumnName);
                    rowInfo.DisplayWidth = ParseStrToInt(xe.GetAttribute(RSDisplayWidth));
                    rowInfo.PickList = xe.GetAttribute(RSPickList);
                    rowInfo.Source = xe.GetAttribute(RSSource);
                    rowInfo.SourceItems = xe.GetAttribute(RSSourceItems);
                    rowInfo.DestItems = xe.GetAttribute(RSDestItems);
                    rowInfo.Reserved1 = xe.GetAttribute(RSReserved1);
                    rowInfo.Reserved2 = xe.GetAttribute(RSReserved2);
                    rowInfo.Reserved3 = xe.GetAttribute(RSReserved3);
                    rowInfo.Reserved4 = xe.GetAttribute(RSReserved4);
                    rowInfo.Reserved5 = xe.GetAttribute(RSReserved5);
                    rowInfo.Reserved6 = xe.GetAttribute(RSReserved6);
                    rowInfo.Reserved7 = xe.GetAttribute(RSReserved7);

                    TableInfo.RowList.Add(rowInfo);
                }
            }

            //indexs
            XmlElement xeIndexs = (XmlElement)xeTable.SelectSingleNode(RSIndexs);
            if (xeIndexs != null)
            {

                XmlNodeList xnlIndexs = xeIndexs.ChildNodes;

                foreach (XmlNode xn in xnlIndexs)
                {
                    XmlElement xe = (XmlElement)xn;

                    ResIndexInfo indexInfo = new ResIndexInfo();

                    indexInfo.Name = xe.GetAttribute(RSIndexName);
                    indexInfo.Valid = ParseStrToBool(xe.GetAttribute(RSValid));
                    indexInfo.Remark = xe.GetAttribute(RSRemark);
                    indexInfo.TableName = xe.GetAttribute(RSTableName);
                    indexInfo.PrimaryKey = ParseStrToBool(xe.GetAttribute(RSPrimaryKey));
                    indexInfo.Constraint = ParseStrToBool(xe.GetAttribute(RSConstraint));
                    indexInfo.Unique = ParseStrToBool(xe.GetAttribute(RSUnique));
                    indexInfo.Clustered = ParseStrToBool(xe.GetAttribute(RSClustered));
                    indexInfo.IndexType = ParseStrToInt(xe.GetAttribute(RSIndexType));
                    indexInfo.AutoUpdate = ParseStrToBool(xe.GetAttribute(RSAutoUpdate));
                    indexInfo.NullCollation = ParseStrToInt(xe.GetAttribute(RSNullCollation));
                    indexInfo.OrdinalPosition = ParseStrToInt(xe.GetAttribute(RSOrdinalPosition));
                    indexInfo.ColumnName = xe.GetAttribute(RSColumnName);
                    indexInfo.Collation = ParseStrToInt(xe.GetAttribute(RSCollation));
                    indexInfo.Cardinality = ParseStrToInt(xe.GetAttribute(RSCardinality));
                    indexInfo.Integrated = ParseStrToBool(xe.GetAttribute(RSIntegrated));

                    TableInfo.IndexList.Add(indexInfo);
                }
            }

            //relations
            XmlElement xeRelations = (XmlElement)xeTable.SelectSingleNode(RSConstraints);
            if (xeRelations != null)
            {

                XmlNodeList xnlRelations = xeRelations.ChildNodes;

                foreach (XmlNode xn in xnlRelations)
                {
                    XmlElement xe = (XmlElement)xn;

                    ResRelationInfo relationInfo = new ResRelationInfo();

                    relationInfo.Name = xe.GetAttribute(RSFKName);
                    relationInfo.Valid = ParseStrToBool(xe.GetAttribute(RSValid));
                    relationInfo.Remark = xe.GetAttribute(RSRemark);
                    relationInfo.TableName = xe.GetAttribute(RSTableName);
                    relationInfo.PKTableName = xe.GetAttribute(RSFKTableName);
                    relationInfo.PKColumnName = xe.GetAttribute(RSPKColumnName);
                    relationInfo.FKTableName = xe.GetAttribute(RSFKTableName);
                    relationInfo.FKColumnName = xe.GetAttribute(RSFKColumnName);
                    relationInfo.Ordinal = ParseStrToInt(xe.GetAttribute(RSOrdinal));
                    relationInfo.UpdateRule = xe.GetAttribute(RSUpdateRule);
                    relationInfo.DeleteRule = xe.GetAttribute(RSDeleteRule);
                    relationInfo.PKName = xe.GetAttribute(RSPKName);
                    relationInfo.Deferrability = ParseStrToInt(xe.GetAttribute(RSDeferrability));

                    TableInfo.RelationList.Add(relationInfo);
                }
            }
        }
    }

}
