{*******************************************************************}
{                                                                   }
{          BDB                                                      }
{          Bain DB Builder                                          }
{                                                                   }
{          author: rain                                             }
{          date created: 2007/09/15                                 }
{          last update: 2007/09/15                                  }
{          remark: N/A                                              }
{                                                                   }
{          Ȩ(c) 2006-2007                            }
{          HTTP://WWW.BAINSOFT.COM                                  }
{          Copyright (c) 2006-2007 BAIN SOFTWARE                    }
{          ALL RIGHTS RESERVED                                      }
{                                                                   }
{*******************************************************************}

unit BDB;

interface

uses
  SysUtils, Classes, RTLConsts;

type
  TaxResItemInfoClass = class of TaxResItemInfo;
  TaxResItemInfo = class
  public
    Name: string;
    Valid: Boolean;
  end;

  TaxResTableHeaderInfo = class(TaxResItemInfo)
  public
    ResIndex: Integer;
    DisplayName: string;
    HeaderSign: string;
    ItemCount: Integer;
    IndexCount: Integer;
    ConstraintCount: Integer;
    PKName: string;
    PK: string;
    Version: string;
    CreateSign: string;
    CreateTime: string;
    LastUpdateSign: string;
    LastUpdateTime: string;

    procedure Clear; virtual;
  end;

  TaxResRowInfo = class(TaxResItemInfo)
  public
    DisplayName: string;
    IsPK: Boolean;
    Index: Integer;
    TypeName: string;
    Precision: Integer;
    Scale: Integer;
    Length: Integer;
    AllowNull: Boolean;
    Identity: Boolean;
    Visible: Boolean;
    Default: string;
    Remark: string;
    OldColumnName: string;
    DisplayWidth: Integer;
    PickList: string;
    Source: string;
    SourceItems: string;
    DestItems: string;
    Reserved1: string;
    Reserved2: string;
    Reserved3: string;
    Reserved4: string;
    Reserved5: string;
    Reserved6: string;
    Reserved7: string;
  end;

  TaxResIndexInfo = class(TaxResItemInfo)
  public
    TableName: string;
    PrimaryKey: Boolean;
    Constraint: Boolean;
    Unique: Boolean;
    Clustered: Boolean;
    IndexType: Integer;
    AutoUpdate: Boolean;
    NullCollation: Integer;
    OrdinalPosition: Integer;
    ColumnName: string;
    Collation: Integer;
    Cardinality: Integer;
    Integrated: Boolean;
    Remark: string;
  end;

  TaxResRelationInfo = class(TaxResItemInfo)
  public
    TableName: string;
    PKTableName: string;
    PKColumnName: string;
    FKTableName: string;
    FKColumnName: string;
    Ordinal: Integer;
    UpdateRule: string;
    DeleteRule: string;
    PKName: string;
    Deferrability: Integer;
    Remark: string;
        
    function RelationUpdate: Boolean;
    function RelationDelete: Boolean;
  end;

  TaxResList = class
  private
    FItemList: TList;
  protected
    function CheckIndex(const AIndex: Integer;
      const ARaiseError: Boolean = True): Boolean;
    function CreateNew: TaxResItemInfo; virtual;
    function GetItems(Index: Integer): TaxResItemInfo; virtual;
    function GetCount: Integer; virtual;
    class function GetResItemInfoClass: TaxResItemInfoClass; virtual;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Clear; virtual;
    function Append: Integer; virtual;
    procedure Delete(const AIndex: Integer); virtual;
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TaxResItemInfo read GetItems;
    property New: TaxResItemInfo read CreateNew;
  end;

  TaxResRowList = class(TaxResList)
  private
    function _GetNew: TaxResRowInfo;
    function _GetItems(Index: Integer): TaxResRowInfo;
  protected
    class function GetResItemInfoClass: TaxResItemInfoClass; override;
  public
    property Items[Index: Integer]: TaxResRowInfo read _GetItems;
    property New: TaxResRowInfo read _GetNew;
  end;

  TaxResIndexList = class(TaxResList)
  private
    function _GetItems(Index: Integer): TaxResIndexInfo;
    function _GetNew: TaxResIndexInfo;
  protected
    class function GetResItemInfoClass: TaxResItemInfoClass; override;
  public
    property Items[Index: Integer]: TaxResIndexInfo read _GetItems;
    property New: TaxResIndexInfo read _GetNew;
  end;

  TaxResRelationList = class(TaxResList)
  private
    function _GetItems(Index: Integer): TaxResRelationInfo;
    function _GetNew: TaxResRelationInfo;
  protected
    class function GetResItemInfoClass: TaxResItemInfoClass; override;
  public
    property Items[Index: Integer]: TaxResRelationInfo read _GetItems;
    property New: TaxResRelationInfo read _GetNew;
  end;

  TaxResTableInfo = class
  private
    FIndexList: TaxResIndexList;
    FRelationList: TaxResRelationList;
    FRowList: TaxResRowList;
    FHeaderInfo: TaxResTableHeaderInfo;
    function GetIndexCount: Integer;
    function GetIndexs(Index: Integer): TaxResIndexInfo;
    function GetRelationCount: Integer;
    function GetRelations(Index: Integer): TaxResRelationInfo;
    function GetRowCount: Integer;
    function GetRows(Index: Integer): TaxResRowInfo;
    function GetTableName: string;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Clear; virtual;
    property TableName: string read GetTableName;
    property HeaderInfo: TaxResTableHeaderInfo read FHeaderInfo;
    property RowList: TaxResRowList read FRowList;
    property IndexList: TaxResIndexList read FIndexList;
    property RelationList: TaxResRelationList read FRelationList;
    property RowCount: Integer read GetRowCount;
    property IndexCount: Integer read GetIndexCount;
    property RelationCount: Integer read GetRelationCount;
    property Rows[Index: Integer]: TaxResRowInfo read GetRows;
    property Indexs[Index: Integer]: TaxResIndexInfo read GetIndexs;
    property Relations[Index: Integer]: TaxResRelationInfo read GetRelations;
  end;

  TaxResController = class
  public
    class function ParseVarToStr(const AValue: OleVariant): string;
    class function ParseStrToBool(const AValue: OleVariant): Boolean;
    class function ParseStrToInt(const AValue: OleVariant): Integer;
    class procedure ResRead(AResFile: string;
      const ATableInfo: TaxResTableInfo); virtual;
  end;

implementation

uses
  XMLDoc, XMLIntf, BDBConsts;

{ TaxResList }

constructor TaxResList.Create;
begin
  inherited;
  FItemList := TList.Create;
end;

destructor TaxResList.Destroy;
begin
  Clear;
  FreeAndNil(FItemList);
  inherited;
end;

function TaxResList.CreateNew: TaxResItemInfo;
begin
  Result := GetResItemInfoClass.Create;
  FItemList.Add(Result);
end;

function TaxResList.Append: Integer;
var
  AItemInfo: TaxResItemInfo;
begin
  AItemInfo := GetResItemInfoClass.Create;
  Result := FItemList.Add(AItemInfo);
end;

procedure TaxResList.Clear;
var
  I: Integer;
  AItemInfo: TaxResItemInfo;
begin
  for I := FItemList.Count-1 downto 0 do
  begin
    AItemInfo := TaxResItemInfo(FItemList.Items[I]);
    if AItemInfo <> nil then
      FreeAndNil(AItemInfo);
    FItemList.Delete(I);
  end;
end;

procedure TaxResList.Delete(const AIndex: Integer);
var
  AItemInfo: TaxResItemInfo;
begin
  CheckIndex(AIndex);
  AItemInfo := GetItems(AIndex);
  if AItemInfo <> nil then
    FreeAndNil(AItemInfo);
  FItemList.Delete(AIndex);
end;

function TaxResList.GetCount: Integer;
begin
  Result := FItemList.Count;
end;

function TaxResList.GetItems(Index: Integer): TaxResItemInfo;
begin
  if not CheckIndex(Index) then
    Result := nil
  else
    Result := TaxResItemInfo(FItemList.Items[Index]);
end;

class function TaxResList.GetResItemInfoClass: TaxResItemInfoClass;
begin
  Result := TaxResItemInfo;
end;

function TaxResList.CheckIndex(const AIndex: Integer;
  const ARaiseError: Boolean): Boolean;
begin
  Result := (AIndex < GetCount) and (AIndex >= 0);
  if not Result and ARaiseError then
    raise EListError.CreateResFmt(@SListIndexError, [AIndex]);
end;

{ TaxResTableHeaderInfo }

procedure TaxResTableHeaderInfo.Clear;
begin
  Name := '';
  DisplayName := '';
  Valid := true;
  HeaderSign := '';
  ItemCount := 0;
  IndexCount := 0;
  ResIndex := -1;
  PKName := '';
  PK := '';
  Version := '';
  CreateSign := '';
  CreateTime := '';
  LastUpdateSign := '';
  LastUpdateTime := '';
end;

{ TaxResRelationInfo }

function TaxResRelationInfo.RelationDelete: Boolean;
begin
  Result := SameText(DeleteRule, RSCascade);
end;

function TaxResRelationInfo.RelationUpdate: Boolean;
begin
  Result := SameText(UpdateRule, RSCascade);
end;

{ TaxResRowList }

function TaxResRowList._GetNew: TaxResRowInfo;
begin
  Result := TaxResRowInfo(inherited New);
end;

function TaxResRowList._GetItems(Index: Integer): TaxResRowInfo;
begin
  Result := TaxResRowInfo(inherited Items[Index]);
end;

class function TaxResRowList.GetResItemInfoClass: TaxResItemInfoClass;
begin
  Result := TaxResRowInfo;
end;

{ TaxResIndexList }

function TaxResIndexList._GetItems(Index: Integer): TaxResIndexInfo;
begin
  Result := TaxResIndexInfo(inherited Items[Index]);
end;

function TaxResIndexList._GetNew: TaxResIndexInfo;
begin
  Result := TaxResIndexInfo(inherited New);
end;

class function TaxResIndexList.GetResItemInfoClass: TaxResItemInfoClass;
begin
  Result := TaxResIndexInfo;
end;

{ TaxResRelationList }

function TaxResRelationList._GetItems(Index: Integer): TaxResRelationInfo;
begin
  Result := TaxResRelationInfo(inherited Items[Index]);
end;

function TaxResRelationList._GetNew: TaxResRelationInfo;
begin
  Result := TaxResRelationInfo(inherited New);
end;

class function TaxResRelationList.GetResItemInfoClass: TaxResItemInfoClass;
begin
  Result := TaxResRelationInfo;
end;

{ TaxResTableInfo }

constructor TaxResTableInfo.Create;
begin
  inherited Create;
  FHeaderInfo := TaxResTableHeaderInfo.Create;
  FRowList := TaxResRowList.Create;
  FIndexList := TaxResIndexList.Create;
  FRelationList := TaxResRelationList.Create;
end;

destructor TaxResTableInfo.Destroy;
begin
  FreeAndNil(FHeaderInfo);
  FreeAndNil(FRowList);
  FreeAndNil(FIndexList);
  FreeAndNil(FRelationList);
  inherited;
end;

procedure TaxResTableInfo.Clear;
begin
  HeaderInfo.Clear;
  RowList.Clear;
  IndexList.Clear;
  RelationList.Clear;
end;

function TaxResTableInfo.GetIndexCount: Integer;
begin
  Result := IndexList.Count;
end;

function TaxResTableInfo.GetIndexs(Index: Integer): TaxResIndexInfo;
begin
  Result := IndexList.Items[Index];
end;

function TaxResTableInfo.GetRelationCount: Integer;
begin
  Result := RelationList.Count;
end;

function TaxResTableInfo.GetRelations(Index: Integer): TaxResRelationInfo;
begin
  Result := RelationList.Items[Index];
end;

function TaxResTableInfo.GetRowCount: Integer;
begin
  Result := RowList.Count;
end;

function TaxResTableInfo.GetRows(Index: Integer): TaxResRowInfo;
begin
  Result := RowList.Items[Index];
end;

function TaxResTableInfo.GetTableName: string;
begin
  Result := HeaderInfo.Name;
end;

{ TaxResController }

class function TaxResController.ParseStrToBool(const AValue: OleVariant): Boolean;
begin
  Result := SameText(ParseVarToStr(AValue), '1');
end;

class function TaxResController.ParseStrToInt(const AValue: OleVariant): Integer;
begin
  Result := StrToIntDef(ParseVarToStr(AValue), 0);
end;

class function TaxResController.ParseVarToStr(
  const AValue: OleVariant): string;
var
  AVType: TVarType;
begin
  AVType := TVarData(AValue).VType;
  if AVType = varNull then
    Result := ''
  else
    Result := AValue;
end;

class procedure TaxResController.ResRead(AResFile: string;
  const ATableInfo: TaxResTableInfo);
var
  I: Integer;
  AXmlDoc: IXMLDocument;
  ARootNode, ATableNode,
  AItemsNode, AItemNode: IXMLNode;
  ANodeList: IXMLNodeList;
  AHeaderInfo: TaxResTableHeaderInfo;
begin
  if (ATableInfo = nil) or not FileExists(AResFile) then
    Exit;
  AXmlDoc := TXMLDOCument.Create(nil);
  try
    AXmlDoc.LoadFromFile(AResFile);
    AXmlDoc.Active := True;
    ARootNode := AXmlDoc.DocumentElement;
    if ARootNode = nil then
      Exit;
      
    ATableNode := ARootNode.ChildNodes[RSTable];
    if ATableNode = nil then
      Exit;

    //header info
    AHeaderInfo := ATableInfo.HeaderInfo;
    with ATableNode, AHeaderInfo do
    begin
      Name := Attributes[RSName];
      Valid := ParseStrToBool(Attributes[RSValid]);
      ResIndex := ParseStrToInt(Attributes[RSResIndex]);
      DisplayName := Attributes[RSDisplayName];
      HeaderSign := Attributes[RSHeaderSign];
      ItemCount := ParseStrToInt(Attributes[RSItemCount]);
      IndexCount := ParseStrToInt(Attributes[RSIndexCount]);
      ConstraintCount := ParseStrToInt(Attributes[RSConstraintCount]);
      PKName := Attributes[RSPKName];
      PK := Attributes[RSPK];
      Version := Attributes[RSVersion];
      CreateSign := Attributes[RSCreateSign];
      CreateTime := Attributes[RSCreateTime];
      LastUpdateSign := Attributes[RSLastUpdateSign];
      LastUpdateTime := Attributes[RSLastUpdateTime];
    end;

    //rows
    AItemsNode := ATableNode.ChildNodes[RSRows];
    if AItemsNode <> nil then
    begin
      ANodeList := AItemsNode.ChildNodes;
      for I := 0 to ANodeList.Count-1 do
      begin
        AItemNode := ANodeList.Nodes[I];
        with AItemNode, ATableInfo.RowList.New do
        begin
          Name := Attributes[RSName];
          Valid := ParseStrToBool(Attributes[RSValid]);
          Visible := ParseStrToBool(Attributes[RSVisible]);
          Remark := Attributes[RSRemark];
          IsPK := ParseStrToBool(Attributes[RSIsPK]);
          Index := ParseStrToInt(Attributes[RSIndex]);
          TypeName := Attributes[RSTypeName];
          Precision := ParseStrToInt(Attributes[RSPrecision]);
          Length := ParseStrToInt(Attributes[RSLength]);
          AllowNull := ParseStrToBool(Attributes[RSAllowNull]);
          Identity := ParseStrToBool(Attributes[RSIdentity]);
          Default := Attributes[RSDefault];
          OldColumnName := Attributes[RSOldColumnName];
          DisplayWidth := ParseStrToInt(Attributes[RSDisplayWidth]);
          PickList := Attributes[RSPickList];
          Source := Attributes[RSSource];
          SourceItems := Attributes[RSSourceItems];
          DestItems := Attributes[RSDestItems];
          Reserved1 := Attributes[RSReserved1];
          Reserved2 := Attributes[RSReserved2];
          Reserved3 := Attributes[RSReserved3];
          Reserved4 := Attributes[RSReserved4];
          Reserved5 := Attributes[RSReserved5];
          Reserved6 := Attributes[RSReserved6];
          Reserved7 := Attributes[RSReserved7];
        end;
      end;
    end;

    //indexs
    AItemsNode := ATableNode.ChildNodes[RSIndexs];
    if AItemsNode <> nil then
    begin
      ANodeList := AItemsNode.ChildNodes;
      for I := 0 to ANodeList.Count-1 do
      begin
        AItemNode := ANodeList.Nodes[I];
        with AItemNode, ATableInfo.IndexList.New do
        begin
          Name := Attributes[RSIndexName];
          Valid := ParseStrToBool(Attributes[RSValid]);
          Remark := Attributes[RSRemark];
          TableName := Attributes[RSTableName];
          PrimaryKey := ParseStrToBool(Attributes[RSPrimaryKey]);
          Constraint := ParseStrToBool(Attributes[RSConstraint]);
          Unique := ParseStrToBool(Attributes[RSUnique]);
          Clustered := ParseStrToBool(Attributes[RSClustered]);
          IndexType := ParseStrToInt(Attributes[RSIndexType]);
          AutoUpdate := ParseStrToBool(Attributes[RSAutoUpdate]);
          NullCollation := ParseStrToInt(Attributes[RSNullCollation]);
          OrdinalPosition := ParseStrToInt(Attributes[RSOrdinalPosition]);
          ColumnName := Attributes[RSColumnName];
          Collation := ParseStrToInt(Attributes[RSCollation]);
          Cardinality := ParseStrToInt(Attributes[RSCardinality]);
          Integrated := ParseStrToBool(Attributes[RSIntegrated]);
        end;
      end;
    end;

    //relations
    AItemsNode := ATableNode.ChildNodes[RSConstraints];
    if AItemsNode <> nil then
    begin
      ANodeList := AItemsNode.ChildNodes;
      for I := 0 to ANodeList.Count-1 do
      begin
        AItemNode := ANodeList.Nodes[I];
        with AItemNode, ATableInfo.RelationList.New do
        begin
          Name := Attributes[RSFKName];
          Valid := ParseStrToBool(Attributes[RSValid]);
          Remark := Attributes[RSRemark];
          TableName := Attributes[RSTableName];
          PKTableName := Attributes[RSFKTableName];
          PKColumnName := Attributes[RSPKColumnName];
          FKTableName := Attributes[RSFKTableName];
          FKColumnName := Attributes[RSFKColumnName];
          Ordinal := ParseStrToInt(Attributes[RSOrdinal]);
          UpdateRule := Attributes[RSUpdateRule];
          DeleteRule := Attributes[RSDeleteRule];
          PKName := Attributes[RSPKName];
          Deferrability := ParseStrToInt(Attributes[RSDeferrability]);
        end;
      end;
    end;
  finally
    FreeAndNil(AXmlDoc);
  end;
end;

end.

